@@ -15,7 +15,8 @@ logger = logging.getLogger(__name__)
 _fixed_public_key = "wt8[6a8eoXFRVL<l2JBbOzs(hcI%kRBIr0Do/eLC"
 _fixed_private_key = "=@e7WwVuz{*eGcnv{AL@x2hmX!z^)wP3vKsQ{S7s"
 
-default_zmq_server_address = "tcp://localhost:60615"
+default_zmq_control_address_for_server = "tcp://*:60615"
+default_zmq_control_address = "tcp://localhost:60615"
 
 
 class CommTimeoutError(TimeoutError):
@@ -398,6 +399,7 @@ class PipeJsonRpcSendAsync:
                 else:
                     # Accept the message. Otherwise wait for timeout
                     self._fut_comm.set_result(response)
+                    self._expected_msg_id = None
             else:
                 # Missing ID: ignore the message
                 logger.error("Received response with missing message ID: %s", pprint.pformat(response))
@@ -430,7 +432,7 @@ class PipeJsonRpcSendAsync:
 #                      Generation and validation of ZMQ key pairs
 
 
-def generate_new_zmq_key_pair():
+def generate_zmq_keys():
     """
     Generates new public-private key pair for ZMQ encryption.
 
@@ -558,7 +560,7 @@ class ZMQCommSendThreads:
         if server_public_key is not None:
             validate_zmq_key(server_public_key)
 
-        zmq_server_address = zmq_server_address or default_zmq_server_address
+        zmq_server_address = zmq_server_address or default_zmq_control_address
         self._server_public_key = server_public_key
 
         self._timeout_receive = timeout_recv  # Timeout for 'recv' operation (ms)
@@ -895,7 +897,7 @@ class ZMQCommSendAsync:
     ):
         self._loop = loop if loop else asyncio.get_event_loop()
 
-        zmq_server_address = zmq_server_address or default_zmq_server_address
+        zmq_server_address = zmq_server_address or default_zmq_control_address
         self._server_public_key = server_public_key
 
         self._timeout_receive = timeout_recv  # Timeout for 'recv' operation (ms)
@@ -1049,6 +1051,9 @@ def zmq_single_request(method, params=None, *, zmq_server_address=None, server_p
     params: dict or None
         Dictionary of parameters (payload of the message). If ``None`` then
         the message is sent with empty payload: ``params = {}``.
+    zmq_server_address: str or None
+        Address of the ZMQ control socket of RE Manager. Default address is used if the
+        value is ``None``.
     server_public_key: str or None
         Server public key (z85-encoded 40 character string). The Valid public key from the server
         public/private key pair must be passed if encryption is enabled at the 0MQ server side.
