@@ -7,9 +7,13 @@ from importlib.util import find_spec
 
 from .worker import RunEngineWorker
 from .manager import RunEngineManager
-from .comms import PipeJsonRpcReceive, validate_zmq_key
+from .comms import PipeJsonRpcReceive, validate_zmq_key, default_zmq_control_address_for_server
 from .profile_ops import get_default_startup_dir
-from .output_streaming import PublishConsoleOutput, setup_console_output_redirection
+from .output_streaming import (
+    PublishConsoleOutput,
+    setup_console_output_redirection,
+    default_zmq_info_address_for_server,
+)
 from .logging_setup import setup_loggers
 
 from .. import __version__
@@ -203,9 +207,10 @@ class WatchdogProcess:
 def start_manager():
 
     s_enc = (
-        "Encryption for ZeroMQ communication server may be enabled by setting QSERVER_ZMQ_PRIVATE_KEY\n"
-        "environment variable to a valid private key (z85-encoded 40 character string):\n\n"
-        "    export QSERVER_ZMQ_PRIVATE_KEY='<private_key>'\n\n"
+        "Encryption for ZeroMQ communication server may be enabled by setting the value of\n"
+        "'QSERVER_ZMQ_PRIVATE_KEY_FOR_SERVER' environment variable to a valid private key\n"
+        "(z85-encoded 40 character string):\n\n"
+        "    export QSERVER_ZMQ_PRIVATE_KEY_FOR_SERVER='<private_key>'\n\n"
         "A public/private key pair may be generated by running 'qserver-zmq-keys'. If RE Manager is\n"
         "configured to use encrypted channel, the encryption must also be enabled at the client side\n"
         "using the public key from the generated pair. Encryption is disabled by default."
@@ -219,12 +224,22 @@ def start_manager():
         description=f"Start Run Engine (RE) Manager\nbluesky-queueserver version {__version__}\n\n{s_enc}",
         formatter_class=formatter,
     )
+    parser.add_argument(
+        "--zmq-control-addr",
+        dest="zmq_control_addr",
+        type=str,
+        default=None,
+        help="The address of ZMQ server (control connection). The parameter overrides the address defined by "
+        "the environment variable QSERVER_ZMQ_CONTROL_ADDRESS_FOR_SERVER. The default address is used if "
+        "the parameter or the environment variable is not defined. Address format: 'tcp://*:60615' "
+        f"(default: {default_zmq_control_address_for_server!r}).",
+    )
     parser.add_argument(
         "--zmq-addr",
         dest="zmq_addr",
         type=str,
-        default="tcp://*:60615",
-        help="The address of ZMQ server (control connection), e.g. 'tcp://*:60615' (default: %(default)s).",
+        default=None,
+        help="The parameter is deprecated and will be removed in future releases. Use --zmq-control-addr instead.",
     )
 
     group = parser.add_mutually_exclusive_group()
@@ -376,13 +391,25 @@ def start_manager():
         "and enable/disable printing and/or publishing of the console output.",
     )
 
+    group_console_output.add_argument(
+        "--zmq-info-addr",
+        dest="zmq_info_addr",
+        type=str,
+        default=None,
+        help="The address of ZMQ server socket used for publishing information on the state of RE Manager "
+        "and currently running processes. Currently only the captured STDOUT and STDERR published "
+        "in 'QS_Console' topic. The parameter overrides the address defined by the environment variable "
+        "'QSERVER_ZMQ_INFO_ADDRESS_FOR_SERVER'. The default address is used if the parameter or the environment "
+        " variable is not defined. Address format: 'tcp://*:60625' "
+        f"(default: {default_zmq_info_address_for_server}).",
+    )
+
     group_console_output.add_argument(
         "--zmq-publish-console-addr",
         dest="zmq_publish_console_addr",
         type=str,
-        default="tcp://*:60625",
-        help="The address of ZMQ server (stdout and stderr publishing), e.g. 'tcp://*:60625' "
-        "(default: %(default)s).",
+        default=None,
+        help="The parameter is deprecated and will be removed in future releases. Use --zmq-info-addr instead.",
     )
 
     group_console_output.add_argument(
@@ -443,7 +470,16 @@ def start_manager():
 
     console_output_on = args.console_output == "ON"
     zmq_publish_console_on = args.zmq_publish_console == "ON"
-    zmq_publish_console_addr = args.zmq_publish_console_addr
+
+    zmq_info_addr = args.zmq_info_addr
+    if args.zmq_publish_console_addr is not None:
+        logger.warning(
+            "Parameter --zmq-publish-console-addr is deprecated and will be removed in future releases. "
+            "Use --zmq-info-addr instead."
+        )
+    zmq_info_addr = zmq_info_addr or args.zmq_publish_console_addr
+    zmq_info_addr = zmq_info_addr or os.environ.get("QSERVER_ZMQ_INFO_ADDRESS_FOR_SERVER", None)
+    zmq_info_addr = zmq_info_addr or default_zmq_info_address_for_server
 
     msg_queue = Queue()
     setup_console_output_redirection(msg_queue)
@@ -455,7 +491,7 @@ def start_manager():
         msg_queue=msg_queue,
         console_output_on=console_output_on,
         zmq_publish_on=zmq_publish_console_on,
-        zmq_publish_addr=zmq_publish_console_addr,
+        zmq_publish_addr=zmq_info_addr,
     )
 
     if zmq_publish_console_on:
@@ -601,8 +637,14 @@ def start_manager():
     config_manager["user_group_permissions_reload"] = args.user_group_permissions_reload
 
     # Read private key from the environment variable, then check if the CLI parameter exists
-    zmq_private_key = os.environ.get("QSERVER_ZMQ_PRIVATE_KEY", None)
-    zmq_private_key = zmq_private_key if zmq_private_key else None  # Case of key==""
+    zmq_private_key = os.environ.get("QSERVER_ZMQ_PRIVATE_KEY_FOR_SERVER", None)
+    if (zmq_private_key is None) and ("QSERVER_ZMQ_PRIVATE_KEY" in os.environ):
+        logger.warning(
+            "Environment variable QSERVER_ZMQ_PRIVATE_KEY is deprecated and will be removed "
+            "in future releases. Use QSERVER_ZMQ_PRIVATE_KEY_FOR_SERVER instead"
+        )
+    zmq_private_key = zmq_private_key or os.environ.get("QSERVER_ZMQ_PRIVATE_KEY", None)
+    zmq_private_key = zmq_private_key or None  # Case of key==""
     if zmq_private_key is not None:
         try:
             validate_zmq_key(zmq_private_key)
@@ -610,7 +652,16 @@ def start_manager():
             logger.error("ZMQ private key is improperly formatted: %s", str(ex))
             return 1
 
-    config_manager["zmq_addr"] = args.zmq_addr
+    zmq_control_addr = args.zmq_control_addr
+    if args.zmq_addr is not None:
+        logger.warning(
+            "Parameter --zmq-addr is deprecated and will be removed in future releases. "
+            "Use --zmq-control-addr instead."
+        )
+    zmq_control_addr = zmq_control_addr or args.zmq_addr
+    zmq_control_addr = zmq_control_addr or os.environ.get("QSERVER_ZMQ_CONTROL_ADDRESS_FOR_SERVER")
+    zmq_control_addr = zmq_control_addr or default_zmq_control_address_for_server
+    config_manager["zmq_addr"] = zmq_control_addr
     config_manager["zmq_private_key"] = zmq_private_key
 
     redis_addr = args.redis_addr
