@@ -5,7 +5,7 @@ import os
 import yaml
 
 from bluesky_queueserver.manager.profile_ops import gen_list_of_plans_and_devices
-from bluesky_queueserver.manager.comms import generate_new_zmq_key_pair
+from bluesky_queueserver.manager.comms import generate_zmq_keys
 
 from .common import (
     patch_first_startup_file,
@@ -22,6 +22,7 @@ from .common import (
     append_code_to_last_startup_file,
     set_qserver_zmq_public_key,
     zmq_single_request,
+    set_qserver_zmq_address,
 )
 
 from .common import re_manager, re_manager_pc_copy, re_manager_cmd  # noqa: F401
@@ -318,20 +319,21 @@ def test_qserver_re_pause_continue(re_manager, option_pause, option_continue):
         assert wait_for_condition(time=3, condition=condition_manager_idle)
 
         n_plans, is_plan_running, n_history = get_reduced_state_info()
-        assert n_plans == 2, "Incorrect number of plans in the queue"
+        assert n_plans == (1 if option_continue == "stop" else 2), "Incorrect number of plans in the queue"
         assert is_plan_running is False
         assert n_history == 1
 
         assert subprocess.call(["qserver", "queue", "start"]) == SUCCESS
 
-        n_history_expected = 3  # Includes entry related to 1 stopped plan
+        # Includes entry related to 1 aborted or halted plan
+        n_history_expected = 2 if option_continue == "stop" else 3
 
     ttime.sleep(1)
 
     n_plans, is_plan_running, n_history = get_reduced_state_info()
-    assert n_plans == 1, "Incorrect number of plans in the queue"
+    assert n_plans == (0 if option_continue == "stop" else 1), "Incorrect number of plans in the queue"
     assert is_plan_running is True
-    assert n_history == n_history_expected - 2
+    assert n_history == n_history_expected - (1 if option_continue == "stop" else 2)
 
     assert wait_for_condition(
         time=60, condition=condition_queue_processing_finished
@@ -1337,7 +1339,7 @@ def test_qserver_secure_1(monkeypatch, re_manager_cmd, test_mode):  # noqa: F811
     Test operation of `qserver` CLI tool with enabled encryption. Test options to
     set the private key used by `qserver` using the environment variable.
     """
-    public_key, private_key = generate_new_zmq_key_pair()
+    public_key, private_key = generate_zmq_keys()
 
     if test_mode == "none":
         pass
@@ -1345,7 +1347,7 @@ def test_qserver_secure_1(monkeypatch, re_manager_cmd, test_mode):  # noqa: F811
         # Set server public key (for 'qserver') using environment variable
         monkeypatch.setenv("QSERVER_ZMQ_PUBLIC_KEY", public_key)
         # Set private key for RE manager
-        monkeypatch.setenv("QSERVER_ZMQ_PRIVATE_KEY", private_key)
+        monkeypatch.setenv("QSERVER_ZMQ_PRIVATE_KEY_FOR_SERVER", private_key)
         # Set public key used by test helper functions such as 'wait_for_condition'
         set_qserver_zmq_public_key(monkeypatch, server_public_key=public_key)
     else:
@@ -1391,6 +1393,52 @@ def test_qserver_secure_1(monkeypatch, re_manager_cmd, test_mode):  # noqa: F811
     ), "Timeout while waiting for environment to be closed"
 
 
+# fmt: off
+@pytest.mark.parametrize("test_mode", ["none", "parameter", "env_var", "both_success", "both_fail"])
+# fmt: on
+def test_qserver_parameters_1(monkeypatch, re_manager_cmd, test_mode):  # noqa: F811
+    """
+    Check that passing server control address as a parameter ``--zmq-control-addr`` and
+    environment variable ``QSERVER_ZMQ_CONTROL_ADDRESS`` works as expected.
+    """
+    address_server = "tcp://*:60621"
+    address_client = "tcp://localhost:60621"
+    address_client_incorrect = "tcp://localhost:60620"
+
+    params_server = [f"--zmq-control-addr={address_server}"]
+    params_client = []
+    if test_mode == "none":
+        # Use default address, communication fails
+        result = COM_ERROR
+    elif test_mode == "parameter":
+        # Pass the address as a parameter
+        result = SUCCESS
+        params_client.append(f"--zmq-control-addr={address_client}")
+    elif test_mode == "env_var":
+        # Pass the address as an environment variable
+        result = SUCCESS
+        monkeypatch.setenv("QSERVER_ZMQ_CONTROL_ADDRESS", address_client)
+    elif test_mode == "both_success":
+        # Pass the correct address as a parameter and incorrect as environment variable (ignored)
+        result = SUCCESS
+        params_client.append(f"--zmq-control-addr={address_client}")
+        monkeypatch.setenv("QSERVER_ZMQ_CONTROL_ADDRESS", address_client_incorrect)
+    elif test_mode == "both_fail":
+        # Pass incorrect address as an environment variable (ignored) and correct address as a parameter
+        result = COM_ERROR
+        params_client.append(f"--zmq-control-addr={address_client_incorrect}")
+        monkeypatch.setenv("QSERVER_ZMQ_CONTROL_ADDRESS", address_client)
+    else:
+        raise RuntimeError(f"Unrecognized test mode '{test_mode}'")
+
+    set_qserver_zmq_address(monkeypatch, zmq_server_address=address_client)
+
+    # Security enabled by setting
+    re_manager_cmd(params_server)
+
+    assert subprocess.call(["qserver", "status"] + params_client) == result
+
+
 # ================================================================================
 #                            qserver-zmq-keys
 
@@ -1406,6 +1454,6 @@ def test_qserver_zmq_keys():
     assert subprocess.call(["qserver-zmq-keys", "--zmq-private-key", "abc"]) == EXCEPTION_OCCURRED
 
     # Generated public key based on private key - success
-    _, private_key = generate_new_zmq_key_pair()
+    _, private_key = generate_zmq_keys()
     print(f"Private key used for the test: '{private_key}'")
     assert subprocess.call(["qserver-zmq-keys", f"--zmq-private-key={private_key}"]) == SUCCESS
