@@ -46,6 +46,24 @@ class EState(enum.Enum):
     CLOSED = "closed"  # For completeness
 
 
+class PlanExecOption(enum.Enum):
+    NEW = "new"  # New plan
+    RESUME = "resume"  # Resume paused plan
+    STOP = "stop"  # Stop paused plan (successful completion)
+    ABORT = "abort"  # Abort paused plan (failed status)
+    HALT = "halt"  # Halt paused plan (failed status, panicked state of RE)
+
+
+# Expected exit status for supported plan execution options (plans could be paused or fail)
+_plan_exit_status_expected = {
+    PlanExecOption.NEW: "completed",
+    PlanExecOption.RESUME: "completed",
+    PlanExecOption.STOP: "stopped",
+    PlanExecOption.ABORT: "aborted",
+    PlanExecOption.HALT: "halted",
+}
+
+
 class RejectedError(RuntimeError):
     ...
 
@@ -142,7 +160,7 @@ class RunEngineWorker(Process):
 
         self._re_namespace, self._plans_in_nspace, self._devices_in_nspace = {}, {}, {}
 
-    def _execute_plan(self, plan, is_resuming):
+    def _execute_plan(self, plan, plan_exec_option):
         """
         Start Run Engine to execute a plan
 
@@ -151,10 +169,8 @@ class RunEngineWorker(Process):
         plan: function
             Reference to a function that calls Run Engine. Run Engine may be called to execute,
             resume, abort, stop or halt the plan. The function should not accept any arguments.
-        is_resuming: bool
-            A flag indicates if the plan is going to be resumed (executed). It is True if
-            'plan' starts a new plan or resumes paused plan. It is False if paused plan is
-            aborted, stopped or halted.
+        plan_exec_option: PlanExecOption
+            Execution option for the plan. See ``PlanExecOption`` for available options.
         """
         logger.debug("Starting execution of a plan")
         try:
@@ -165,13 +181,14 @@ class RunEngineWorker(Process):
                     "success": True,
                     "result": result,
                     "err_msg": "",
+                    "traceback": "",
                 }
-                if is_resuming:
+                if plan_exec_option in (PlanExecOption.NEW, PlanExecOption.RESUME):
                     self._re_report["plan_state"] = "completed"
                     self._running_plan_completed = True
                 else:
                     # Here we don't distinguish between stop/abort/halt
-                    self._re_report["plan_state"] = "stopped"
+                    self._re_report["plan_state"] = _plan_exit_status_expected[plan_exec_option]
                     self._running_plan_completed = True
 
                 # Include RE state
@@ -180,30 +197,36 @@ class RunEngineWorker(Process):
                 # Clear the list of active runs
                 self._active_run_list.clear()
 
+                logger.info("The plan was exited. Plan state: %s", self._re_report["plan_state"])
+
         except BaseException as ex:
             with self._re_report_lock:
 
                 self._re_report = {
                     "action": "plan_exit",
                     "result": "",
-                    "err_msg": str(ex),
+                    "traceback": traceback.format_exc(),
                 }
 
                 if self._RE._state == "paused":
                     # Run Engine was paused
                     self._re_report["plan_state"] = "paused"
                     self._re_report["success"] = True
+                    self._re_report["err_msg"] = "The plan is paused"
+                    logger.info("The plan is paused ...")
 
                 else:
                     # RE crashed. Plan execution can not be resumed. (Environment may have to be restarted.)
                     # TODO: clarify how this situation must be handled. Also additional error handling
                     #       may be required
-                    self._re_report["plan_state"] = "error"
+                    self._re_report["plan_state"] = "failed"
                     self._re_report["success"] = False
+                    self._re_report["err_msg"] = f"Plan failed: {ex}"
                     self._running_plan_completed = True
 
                     # Clear the list of active runs (don't clean the list for the paused plan).
                     self._active_run_list.clear()
+                    logger.error("The plan failed: %s", self._re_report["err_msg"])
 
                 # Include RE state
                 self._re_report["re_state"] = str(self._RE._state)
@@ -276,16 +299,20 @@ class RunEngineWorker(Process):
             # 'is_resuming' is true (we start a new plan that is supposedly runs to completion
             #   as opposed to aborting/stopping/halting a plan)
         except Exception as ex:
+            logger.exception(ex)
+
             # We want the exception to be raised in the main thread (plan execution)
-            def get_plan(err_msg):
+            def get_plan(ex):
                 def plan():
-                    raise Exception(err_msg)
+                    raise RuntimeError(
+                        "Error occurred while processing plan parameters or starting the plan"
+                    ) from ex
 
-                return plan()
+                return plan
 
-            plan = get_plan(str(ex))
+            plan = get_plan(ex)
 
-        self._execution_queue.put((plan, True))
+        self._execution_queue.put((plan, PlanExecOption.NEW))
 
     def _continue_plan(self, option):
         """
@@ -315,7 +342,7 @@ class RunEngineWorker(Process):
                 if self._RE._state == "panicked":
                     raise RuntimeError(
                         "Run Engine is in the 'panicked' state. "
-                        "You need to recreate the environment before you can run plans."
+                        "The worker environment must be closed and reopened before plans could be executed."
                     )
                 elif self._RE._state != "paused":
                     raise RuntimeError(
@@ -332,8 +359,7 @@ class RunEngineWorker(Process):
             return plan
 
         plan = get_plan(option, available_options)
-        is_resuming = option == "resume"
-        self._execution_queue.put((plan, is_resuming))
+        self._execution_queue.put((plan, PlanExecOption(option)))
 
     def _generate_lists_of_allowed_plans_and_devices(self):
         """
@@ -628,11 +654,11 @@ class RunEngineWorker(Process):
             try:
                 s = msg_list[invalid_state - 1]
             except Exception:
-                s = "UNDETERMINED CONDITION IS PRESENT"  # Shouldn't ever be printed
+                s = "UNKNOWN CONDITION IS DETECTED IN THE WORKER PROCESS"  # Shouldn't ever be printed
             err_msg = (
-                f"Trying to run a plan (start Run Engine) while {s}.\n"
+                f"Attempt to start a plan (Run Engine) while {s}.\n"
                 "This may indicate a serious issue with the plan queue execution mechanism.\n"
-                "Please report the issue to developers."
+                "Please report the issue to developer team."
             )
 
         msg_out = {"status": status, "err_msg": err_msg}
@@ -773,8 +799,8 @@ class RunEngineWorker(Process):
             if self._exit_event.is_set():
                 break
             try:
-                plan, is_resuming = self._execution_queue.get(False)
-                self._execute_plan(plan, is_resuming)
+                plan, plan_exec_option = self._execution_queue.get(False)
+                self._execute_plan(plan, plan_exec_option)
             except queue.Empty:
                 pass
 
@@ -825,11 +851,17 @@ class RunEngineWorker(Process):
                         except Exception as ex_json:
                             raise ValueError(f"Task result can not be serialized as JSON: {ex_json}") from ex_json
 
-                        success, msg = True, ""
+                        success, err_msg, err_tb = True, "", ""
                     except Exception as ex:
-                        logger.exception("Error occurred while executing the function ('%s'): %s", name, str(ex))
-                        return_value = traceback.format_exc()
-                        success, msg = False, f"Exception: {str(ex)}"
+                        s = f"Error occurred while executing {name!r}"
+                        err_msg = f"{s}: {str(ex)}"
+                        if hasattr(ex, "tb"):  # ScriptLoadingError
+                            err_tb = str(ex.tb)
+                        else:
+                            err_tb = traceback.format_exc()
+                        logger.error("%s:\n%s\n", err_msg, err_tb)
+
+                        return_value, success = None, False
                     finally:
                         if not run_in_background:
                             self._env_state = EState.IDLE
@@ -841,7 +873,8 @@ class RunEngineWorker(Process):
                         task_res = {
                             "task_uid": task_uid,
                             "success": success,
-                            "msg": msg,
+                            "msg": err_msg,
+                            "traceback": err_tb,
                             "return_value": return_value,
                             "time_start": time_start,
                             "time_stop": ttime.time(),
@@ -987,10 +1020,11 @@ class RunEngineWorker(Process):
             logger.info("Startup code loading was completed")
 
         except Exception as ex:
-            logger.exception(
-                "Failed to start RE Worker environment. Error while loading startup code: %s.",
-                str(ex),
-            )
+            s = "Failed to start RE Worker environment. Error while loading startup code"
+            if hasattr(ex, "tb"):  # ScriptLoadingError
+                logger.error("%s:\n%s\n", s, str(ex.tb))
+            else:
+                logger.exception("%s: %s.", s, str(ex))
             success = False
 
         if success:
